/// <reference path="../reference/cristalix.d.ts" />
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as easing from './easing';
// import * as renderhelper from './renderhelper';
var pushMatrix = GL11.glPushMatrix;
var popMatrix = GL11.glPopMatrix;
var translate = GL11.glTranslatef;
var rotate = GL11.glRotatef;
var scale = GL11.glScalef;
var depthMask = GL11.glDepthMask;
var color = GL11.glColor4f;
export var CENTER = { x: 0.5, y: 0.5 };
export var LEFT = { x: 0, y: 0.5 };
export var RIGHT = { x: 1, y: 0.5 };
export var TOP = { x: 0.5, y: 0 };
export var BOTTOM = { x: 0.5, y: 1 };
export var TOP_RIGHT = { x: 1, y: 0 };
export var TOP_LEFT = { x: 0, y: 0 };
export var BOTTOM_RIGHT = { x: 1, y: 1 };
export var BOTTOM_LEFT = { x: 0, y: 1 };
var Animatable = /** @class */ (function () {
    function Animatable(value) {
        this.value = value;
        this.started = 0;
        this.duration = 0;
        this.fromValue = 0;
        this.toValue = 0;
        this.easer = easing.none;
        this.onFinish = null;
        this.toValue = value;
    }
    Animatable.prototype.transit = function (value, duration, easer, onFinish) {
        this.fromValue = this.value;
        this.toValue = value;
        this.started = getTime();
        this.duration = duration;
        this.easer = easer;
        this.onFinish = onFinish;
    };
    Animatable.prototype.update = function (time) {
        if (!this.started)
            return;
        var part = (time - this.started) / this.duration;
        if (part > 1.0) {
            this.value = this.toValue;
            this.started = 0.0;
            var finish = this.onFinish;
            this.onFinish = null;
            if (finish != null)
                finish();
        }
        else {
            part = this.easer(part);
            this.value = this.fromValue + (this.toValue - this.fromValue) * part;
        }
    };
    return Animatable;
}());
export { Animatable };
export function color2Hex(color) {
    return colorParts2Hex(color.a, color.r, color.g, color.b);
}
export function colorParts2Hex(a, r, g, b) {
    return a * 255 << 24 |
        r * 255 << 16 |
        g * 255 << 8 |
        b * 255;
}
export function hex2Color(value, includeAlpha) {
    var a = includeAlpha ? ((value >> 24) & 0xFF) / 255.0 : 1;
    var r = ((value >> 16) & 0xFF) / 255.0;
    var g = ((value >> 8) & 0xFF) / 255.0;
    var b = (value & 0xFF) / 255.0;
    return { a: a, r: r, g: g, b: b };
}
export function getTime() {
    return System.currentTimeMillis();
}
var el;
el = { x: 1, color: {} };
var Element = /** @class */ (function () {
    function Element(data) {
        this.x = new Animatable(data.x || 0);
        this.y = new Animatable(data.y || 0);
        this.z = new Animatable(data.z || 0);
        var color = data.color || {};
        this.a = new Animatable(color.a == null ? 1 : color.a);
        this.r = new Animatable(color.r == null ? 0 : color.r);
        this.g = new Animatable(color.g == null ? 0 : color.g);
        this.b = new Animatable(color.b == null ? 0 : color.b);
        this.scale = new Animatable(data.scale == null ? 1 : data.scale);
        var align = data.align || { x: 0, y: 0 };
        this.alignX = new Animatable(align.x || 0);
        this.alignY = new Animatable(align.y || 0);
        var origin = data.origin || { x: 0, y: 0 };
        this.originX = new Animatable(origin.x || 0);
        this.originY = new Animatable(origin.y || 0);
        this.rotationX = new Animatable(data.rotationX || 0);
        this.rotationY = new Animatable(data.rotationY || 0);
        this.rotationZ = new Animatable(data.rotationZ || 0);
        this.enabled = data.enabled == null ? true : data.enabled;
        this.noDepth = !!data.noDepth;
        this.animatables = [
            this.x, this.y, this.z,
            this.a, this.r, this.g, this.b,
            this.scale,
            this.rotationX, this.rotationY, this.rotationZ,
            this.alignX, this.alignY,
            this.originX, this.originY,
        ];
    }
    ;
    Element.prototype.prepare = function (time, parentWidth, parentHeight, elementWidth, elementHeight) {
        if (!this.enabled)
            return;
        // this.updateAnimatables(time);
        this.prepareAlign(parentWidth, parentHeight);
        this.prepareRotation();
        this.prepareOffset();
        this.prepareScale();
        this.prepareOrigin(elementWidth, elementHeight);
        this.lastColor = colorParts2Hex(this.a.value, this.r.value, this.g.value, this.b.value);
    };
    Element.prototype.updateAnimatables = function (time) {
        var animatables = this.animatables;
        for (var i = 0; i < animatables.length; i++) {
            var animatable = animatables[i];
            if (animatable.started)
                animatable.update(time);
        }
    };
    Element.prototype.prepareAlign = function (parentWidth, parentHeight) {
        if (this.alignX.value || this.alignY.value)
            translate(parentWidth * this.alignX.value, parentHeight * this.alignY.value, 0);
    };
    Element.prototype.prepareRotation = function () {
        if (this.rotationX.value)
            rotate(this.rotationX.value, 1, 0, 0);
        if (this.rotationY.value)
            rotate(this.rotationY.value, 0, 1, 0);
        if (this.rotationZ.value)
            rotate(this.rotationZ.value, 0, 0, 1);
    };
    Element.prototype.prepareOffset = function () {
        if (this.x.value || this.y.value || this.z.value)
            translate(this.x.value, this.y.value, 0 /*this.z.value*/);
    };
    Element.prototype.prepareScale = function () {
        if (this.scale.value != 1)
            scale(this.scale.value, this.scale.value, this.scale.value);
    };
    Element.prototype.prepareOrigin = function (elementWidth, elementHeight) {
        if (this.originX.value || this.originY.value)
            translate(-elementWidth * this.originX.value, -elementHeight * this.originY.value, 0);
    };
    return Element;
}());
export { Element };
var Hz = /** @class */ (function () {
    function Hz() {
    }
    return Hz;
}());
export { Hz };
export function text(data) {
    return new Text(data);
}
var Text = /** @class */ (function (_super) {
    __extends(Text, _super);
    function Text(data) {
        var _this = _super.call(this, data) || this;
        _this.text = data.text || "";
        _this.shadow = !!data.shadow;
        _this.autoFit = !!data.autoFit;
        return _this;
    }
    Text.prototype.render = function (time, parentWidth, parentHeight) {
        if (!this.enabled)
            return;
        var textWidth = Draw.getStringWidth(this.text);
        var textHeight = 9;
        if (textWidth && this.autoFit) {
            var factor = (parentWidth - 2) / textWidth;
            if (factor < 1)
                this.scale.value = factor;
        }
        pushMatrix();
        _super.prototype.prepare.call(this, time, parentWidth, parentHeight, textWidth, textHeight);
        // GL11.glDepthFunc(GL11.GL_LESS);
        // GlStateManager.disableDepth();
        if (this.scale.value)
            Draw.drawString(this.text, 0, fontRenderer.getUnicodeFlag() ? 0 : 1, -1, this.shadow);
        // GL11.glDepthFunc(GL11.GL_LEQUAL);
        // GlStateManager.enableDepth();
        popMatrix();
    };
    Text.prototype.checkHovered = function (screenState, parentWidth, parentHeight) { };
    return Text;
}(Element));
export { Text };
export function rect(data) {
    return new Box(data);
}
var Box = /** @class */ (function (_super) {
    __extends(Box, _super);
    function Box(data) {
        var _this = _super.call(this, data) || this;
        _this.hovered = false;
        _this.width = new Animatable(data.width || 0);
        _this.height = new Animatable(data.height || 0);
        _this.children = data.children || [];
        _this.texture = data.texture || null;
        _this.onLeftClick = data.onLeftClick || null;
        _this.onRightClick = data.onRightClick || null;
        _this.onHover = data.onHover || null;
        _this.uMin = data.uMin || 0;
        _this.vMin = data.vMin || 0;
        _this.uDelta = data.uDelta || 256;
        _this.vDelta = data.vDelta || 256;
        _this.textureWidth = data.textureWidth || 256;
        _this.textureHeight = data.textureHeight || 256;
        _this.temp_magic = !!data.temp_magic;
        _this.beforeRender = data.beforeRender || null;
        _this.afterRender = data.afterRender || null;
        return _this;
    }
    Box.prototype.render = function (time, parentWidth, parentHeight) {
        if (!this.enabled)
            return;
        this.width.update(time);
        this.height.update(time);
        var width = this.width.value;
        var height = this.height.value;
        pushMatrix();
        _super.prototype.prepare.call(this, time, parentWidth, parentHeight, width, height);
        this.render0(width, height);
        translate(0, 0, 0.01);
        for (var i = 0; i < this.children.length; i++) {
            this.children[i].render(time, width, height);
        }
        popMatrix();
    };
    Box.prototype.render0 = function (width, height) {
        if (this.scale.value) {
            if (this.beforeRender)
                this.beforeRender();
            if (this.texture) {
                GlStateManager.enableBlend();
                Textures.bindTexture(this.texture);
                color(this.a.value, this.r.value, this.g.value, this.b.value);
                Draw.drawScaledCustomSizeModalRect(0, 0, this.uMin, this.vMin, this.uDelta, this.vDelta, width, height, this.textureWidth, this.textureHeight);
            }
            else {
                Draw.drawRect(0, 0, width, height, this.lastColor);
            }
            if (this.afterRender)
                this.afterRender();
        }
    };
    Box.prototype.checkHovered = function (ss, parentWidth, parentHeight) {
        if (!this.enabled)
            return;
        // if (!stack.enabled || !(this.children.length || action)) return;
        var originX = this.originX.value;
        var originY = this.originY.value;
        var alignX = this.alignX.value;
        var alignY = this.alignY.value;
        var width = this.width.value;
        var height = this.height.value;
        var scale = this.scale.value;
        var x = this.x.value;
        var y = this.y.value;
        ss.stackX += parentWidth * alignX * ss.stackScale;
        ss.stackY += parentHeight * alignY * ss.stackScale;
        ss.stackX += x * ss.stackScale;
        ss.stackY += y * ss.stackScale;
        ss.stackScale *= scale;
        ss.stackX -= width * originX * ss.stackScale;
        ss.stackY -= height * originY * ss.stackScale;
        var dx = ss.mouseX - ss.stackX;
        var dy = ss.mouseY - ss.stackY;
        var hovered = dx >= 0 && dx < width * ss.stackScale && dy >= 0 && dy < height * ss.stackScale;
        if (this.hovered != hovered && this.onHover) {
            this.onHover(ss, hovered);
        }
        this.hovered = hovered;
        if (this.hovered) {
            if (ss.leftClick && this.onLeftClick)
                this.onLeftClick(ss);
            if (ss.rightClick && this.onRightClick)
                this.onRightClick(ss);
        }
        for (var _i = 0, _a = this.children; _i < _a.length; _i++) {
            var child = _a[_i];
            child.checkHovered(ss, parentWidth, parentHeight);
        }
        // str = "stack: " + ss.stackX + " " + ss.stackY + " " + ss.stackScale + 
        //  ", d: " + dx + " " + dy + ", pos: " + this.stack.x.value + " " + this.stack.y.value;
        ss.stackX += width * originX * ss.stackScale;
        ss.stackY += height * originY * ss.stackScale;
        ss.stackScale /= scale;
        ss.stackX -= x * ss.stackScale;
        ss.stackY -= y * ss.stackScale;
        ss.stackX -= parentWidth * alignX * ss.stackScale;
        ss.stackY -= parentHeight * alignY * ss.stackScale;
    };
    return Box;
}(Element));
export { Box };
var Item = /** @class */ (function (_super) {
    __extends(Item, _super);
    function Item(data) {
        var _this = _super.call(this, data) || this;
        _this.item = data.item;
        return _this;
    }
    Item.prototype.render = function (time, parentWidth, parentHeight) {
        if (!this.enabled)
            return;
        pushMatrix();
        RenderHelper.enableGUIStandardItemLighting();
        // GL11.glTranslatef(0, 0, +100);
        _super.prototype.prepare.call(this, time, parentWidth, parentHeight, 16, 16);
        Draw.renderItemAndEffectIntoGUI(this.item, this.x.value, this.y.value);
        RenderHelper.disableStandardItemLighting();
        popMatrix();
    };
    Item.prototype.checkHovered = function (screenState, parentWidth, parentHeight) { };
    return Item;
}(Element));
export { Item };
export function getScreenState() {
    var factor = Draw.getResolution().getScaleFactor();
    var screenWidth = Display.getWidth() / factor;
    var screenHeight = Display.getHeight() / factor;
    var mouseX = Mouse.getX() / factor;
    var mouseY = screenHeight - Mouse.getY() / factor;
    var leftClick = Mouse.isButtonDown(0);
    var rightClick = Mouse.isButtonDown(1);
    var time = getTime();
    return {
        factor: factor,
        width: screenWidth,
        height: screenHeight,
        mouseX: mouseX,
        mouseY: mouseY,
        time: time,
        leftClick: leftClick,
        rightClick: rightClick,
        stackX: 0,
        stackY: 0,
        stackScale: 1
    };
}
export var overlay = [];
export var zIndex = 0;
Events.on(plugin, 'gui_overlay_render', function (e) {
    // Draw.drawString(str, 10, 10);
    var screenState = getScreenState();
    translate(0, 0, +zIndex);
    for (var _i = 0, overlay_1 = overlay; _i < overlay_1.length; _i++) {
        var element = overlay_1[_i];
        element.render(screenState.time, screenState.width, screenState.height);
    }
    translate(0, 0, -zIndex);
});
// ToDo: one-time click detection
// const mouseButtons = [
//     {event: 'left_click', pressed: false, property: 'onLeftClick'},
//     {event: 'right_click', pressed: false, property: 'onRightClick'},
// ]
Events.on(plugin, 'game_loop', function (e) {
    var screenState = getScreenState();
    for (var _i = 0, overlay_2 = overlay; _i < overlay_2.length; _i++) {
        var element = overlay_2[_i];
        element.checkHovered(screenState, screenState.width, screenState.height);
    }
    // for (var i = 0; i < mouseButtons.length; i++) {
    //     let button = mouseButtons[i];
    //     if (!Mouse.isButtonDown(i)) {
    //         button.pressed = false;
    //         continue;
    //     }
    //     if (button.pressed) continue;
    //     button.pressed = true;
    //     for (var j = 0; j < root.children.length; j++) {
    //         if (root.children[j].executeHovered) 
    //             root.children[j].executeHovered(screenState, button.property, screenState.width, screenState.height);
    //     }
    //     Events.post(button.event, screenState);
    // }
});
